<?php

namespace FORM4\Form4Teaser\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * API class
 * A simple API for adding, removing and getting data to and from the TYPO3_CONF_VARS array
 *
 * @package TYPO3
 * @subpackage form4_teaser
 *
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 *
 * @api
 *
 * TODO create a report page that shows whether settings are overwritten (because such things are often the root of a problem)
 */
final class ConfigurationUtility {

	const _EXTKEY  = 'form4_teaser';


	/**
	 * Unregister all layouts, sortings and sorting directions.
	 *
	 * @return void
	 */
	public static function reset() {
		$GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY] = array(
			'layouts' => array(),
			'sortings' => array(),
			'sortingDirections' => array()
		);
	}

	/**
	 * Register a fluid layout.
	 *
	 * @param string $key
	 * @param <mixed> $file Path to a Fluid template
	 * @param string $label A simple string or a LLL string
	 * @param <mixed> $icon Path to an icon or FALSE
	 * @return void
	 */
	public static function addFluidLayout($key, $file, $label, $icon = FALSE) {
		self::add('layouts', $key, array(
			'type' => 'fluid',
			'template' => $file,
			'label' => $label,
			'icon' => $icon
		));
	}

	/**
	 * Register a sorting.
	 *
	 * @param string $key
	 * @param <mixed> $funcName A callable or a string referencing a function/method
	 * @param string $label A simple string or a LLL string
	 * @param <mixed> $icon Path to an icon or FALSE
	 * @return void
	 */
	public static function addSorting($key, $funcName, $label, $icon) {
		self::add('sortings', $key, array(
			'userFunction' => $funcName,
			'label' => $label,
			'icon' => $icon
		));
	}

	/**
	 * Register a fluid sorting direction.
	 *
	 * @param string $key
	 * @param <mixed> $funcName A callable or a string referencing a function/method
	 * @param string $label A simple string or a LLL string
	 * @param <mixed> $icon Path to an icon or FALSE
	 * @return void
	 */
	public static function addSortingDirection($key, $funcName, $label, $icon) {
		self::add('sortingDirections', $key, array(
			'userFunction' => $funcName,
			'label' => $label,
			'icon' => $icon
		));
	}


	/**
	 * Remove a layout by its key.
	 *
	 * @param string $key Key of a layout
	 */
	public static function removeLayout($key) {
		self::remove('layouts', $key);
	}

	/**
	 * Remove a sorting by its key.
	 *
	 * @param string $key Key of a sorting
	 */
	public static function removeSorting($key) {
		self::remove('sortings', $key);
	}

	/**
	 * Remove a sorting direction by its key.
	 *
	 * @param string $key Key of a sorting direction
	 */
	public static function removeSortingDirection($key) {
		self::remove('sortingDirections', $key);
	}

	/**
	 * Get a layout by its key or get an array of all layouts.
	 *
	 * @param string $key
	 * @return array When $key is given returns an array containing the registered data of the layout. If $key is NULL an array containing all layouts is returned.
	 */
	public static function getLayout($key = NULL) {
		return self::get('layouts', $key);
	}

	/**
	 * Get a sorting by its key or get an array of all sortings.
	 *
	 * @param string $key
	 * @return array When $key is given returns an array containing the registered data of the sorting. If $key is NULL an array containing all sortings is returned.
	 */
	public static function getSorting($key = NULL) {
		return self::get('sortings', $key);
	}

	/**
	 * Get a sorting direction by its key or get an array of all sorting directions.
	 *
	 * @param string $key
	 * @return array When $key is given returns an array containing the registered data of the sorting direction. If $key is NULL an array containing all sorting directions is returned.
	 */
	public static function getSortingDirection($key = NULL) {
		return self::get('sortingDirections', $key);
	}


	/**
	 * Add a key to the TYPO3_CONF_VARS.
	 *
	 * @param string $type
	 * @param string $key
	 * @param <mixed> $value A primitive data type
	 * @return void
	 */
	private static function add($type, $key, $value) {
		$GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type][$key] = $value;
	}

	/**
	 * Remove a key from the TYPO3_CONF_VARS.
	 *
	 * @param string $type
	 * @param string $key
	 * @return void
	 */
	private static function remove($type, $key) {
		if ($key === NULL) {
			$GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type] = array();
		} else {
			unset($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type][$key]);
		}
	}

	/**
	 * Get a key from the TYPO3_CONF_VARS.
	 *
	 * @param string $type
	 * @param string $key
	 * @return <mixed> A primitive data type
	 */
	private static function get($type, $key) {
		$value = FALSE;
		if ($key === NULL) {
			$value = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type];
		} elseif (array_key_exists($key, $GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type])) {
			$value = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][self::_EXTKEY][$type][$key];
		}
		return $value;
	}

}
?>